/* Arduino morse Encoder
 *  
 *PS2Keyboard library example
  PS2Keyboard now requries both pins specified for begin()
  keyboard.begin(data_pin, irq_pin);

  Valid irq pins:
     Arduino Uno:  2, 3
     Arduino Due:  All pins, except 13 (LED)
     Arduino Mega: 2, 3, 18, 19, 20, 21
     Teensy 3.0:   All pins, except 13 (LED)
     Teensy 2.0:   5, 6, 7, 8
     Teensy 1.0:   0, 1, 2, 3, 4, 6, 7, 16
     Teensy++ 2.0: 0, 1, 2, 3, 18, 19, 36, 37
     Teensy++ 1.0: 0, 1, 2, 3, 18, 19, 36, 37
     Sanguino:     2, 10, 11

  for more information you can read the original wiki in arduino.cc
  at http://www.arduino.cc/playground/Main/PS2Keyboard
  or http://www.pjrc.com/teensy/td_libs_PS2Keyboard.html

  Like the Original library and example this is under LGPL license.

  Modified by Cuninganreset@gmail.com on 2010-03-22
  Modified by Paul Stoffregen <paul@pjrc.com> June 2010
*/

#include <PS2Keyboard.h>
#include <U8g2lib.h>
#include "Lewis.h"
#include <TimerOne.h>


U8G2_ST7565_ERC12864_1_4W_SW_SPI u8g2 ( U8G2_R0, /* scl=*/  13 , /* si=*/  11 , /* cs=*/  10 , /* rs=*/  9 , /* rse=*/  8 ) ;

const int DataPin = 3;
const int IRQpin =  2;

char* znak;
char* wiersz[10];                                 //char workline*
char* wiersz1[10];                                //top line char*
int wiersz_M[10];                                 //workline in ASCII code
int wiersz_M1[10];                                //top line in ASCII code
byte k;
int d, e, i;                                      //helper variables
byte isnm = 7;                                    //number of words per minute
char isnm_str[2];
byte kursor;
const byte index_key = 48;                        //number of characters in the array
const char* KeyChar[index_key] = {
"a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s",
"t","u","v","w","x","y","z","0","1","2","3","4","5","6","7","8","9"," ",
"!","(",")","+",",","-",".","/","=","?","_"};
const int ASCII[index_key] = {
 97, 98, 99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,
116,117,118,119,120,121,122, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 32,
 33, 40, 41, 43, 44, 45, 46, 47, 61, 63, 95};

PS2Keyboard keyboard;
Lewis Morse;


void setup() {
  delay(1000);
  keyboard.begin(DataPin, IRQpin);
  Serial.begin(9600);
 // u8g2.begin ();
 // u8g2.setContrast(20);
  //Morse.begin(rx_pin, tx_pin, words_per_minute, use_interrupts)

  Morse.begin(7,7, isnm, true);
  u8g2.begin ();
  u8g2.setContrast(40);
  //use the TimerOne library for a 100Hz timer interrupt
  Timer1.initialize(1000);
  Timer1.attachInterrupt(MorseISR);
}

void loop() {
  PS2();                                           //reading characters from the keyboard
  if (znak == "enter" ) { Send_Morse();}           //sending the line to broadcast
   Display(); //display the line
  Display();                                       //display the line
}

void PS2() {
  if (keyboard.available()) {
    // read the next key
    char c = keyboard.read();
    d = int(c);
    // check for some of the special keys
    if (c == PS2_ENTER) {
      znak ="enter";
//    } else if (c == PS2_TAB) {
//      Serial.print("[Tab]");
//    } else if (c == PS2_ESC) {
//      Serial.print("[ESC]");
    } else if (c == PS2_PAGEDOWN) {
      isnm--;
      if (isnm < 1 ) {isnm = 1;}
      
      Morse.begin(7,7, isnm, true);
    } else if (c == PS2_PAGEUP) {
      isnm++;
      if (isnm > 30) {isnm = 30;} 
     Morse.begin(7,7, isnm, true);
     
  //  } else if (c == PS2_LEFTARROW) {
  //    Serial.print("[Left]");
  //  } else if (c == PS2_RIGHTARROW) {
  //    Serial.print("[Right]");
  //  } else if (c == PS2_UPARROW) {
  //    Serial.print("[Up]");
  //  } else if (c == PS2_DOWNARROW) {
  //    Serial.print("[Down]");
    } else if (c == PS2_DELETE) {
      znak = "back";
      i--;
      znak = " ";
      wiersz[i] = znak;
      wiersz_M[i] = d;
    } else {
      // otherwise substitute a regular letter
      getKeyChar();                           //converting char to char*
      wiersz[i] = znak;
      wiersz_M[i] = d;
      if (i > 9) {i = -1;}
      if (znak == "enter"){i = -1;}
      if (znak != "back" && znak != "enter") {i++;}     //increase character counter
    }
  }
}

void Display() {                              //display data on the LCD
  kursor++;
  if (kursor >20){ kursor = 0;}               
  sprintf(isnm_str,"%d", isnm);               //converting a number to a string
  u8g2.firstPage();
  do {
    u8g2.setFont(u8g2_font_10x20_tr);
    u8g2.drawStr(0, 15, "Morse");
    u8g2.drawStr(75, 15, isnm_str);
    u8g2.drawStr(98, 15, "w/m");
    for (k = 0; k < 10; k++){  
       u8g2.drawStr(k*10, 40, wiersz1[k] );
       u8g2.drawStr(k*10, 60, wiersz[k] );
    }
    if (kursor<15){u8g2.drawStr(i*10, 60, "_" );}     //display the cursor
    if (kursor>5){u8g2.drawStr(i*10, 60, " " );}      //cursor blinking
  } while ( u8g2.nextPage() );
}

void Send_Morse(){
  znak = " "; 
  for (int k = 0; k < 10; k++) {
     wiersz1[k] = " ";                        //cash
     wiersz1[k] = wiersz[k];                  //rewriting characters from the working line to the top
     wiersz[k] = " ";                         //delete work line
     wiersz_M1[k] = 32;                       //delete ASCII top line
     wiersz_M1[k] = wiersz_M[k];              //rewrite ASCII from the working line to the top line
     wiersz_M[k] = 32;                        //delete ASCII worklineroboczego     
  }  
  for (int k = 1; k < 10; k++){
     e = wiersz_M1[k];
     Morse.write(e);                          //morse the contents of the top line
  }
  i = 0;
}

void MorseISR(){
  Morse.timerISR();                           //function triggered by interrupts (keyboard)
}

void getKeyChar() {                           //converting a key code into a letter and an ASCII code
  for (k = 0; k <= index_key; k++){
     if (ASCII[k] == d) { znak = KeyChar[k];} //convert key code to letter
  }
  k = 0;
}
